/*---------------------------------------------------------------------------
 * Title:	muecke.c
 * Date:		19.05.2014
 * Referenz:Mueckensimulator von Alex S., Juli 2009
 * 			http://www.mikrocontroller.net/topic/145156
 *
 * Atmel Microprocessor ATtiny13
 * lfuse = 0x7A
 * hfuse = 0xFF
 *              ATtiny13
 *    Vcc o   +----v----+     o Vcc (+2.4V .. +5V)
 *        |   |         |     |
 *        +---+ 1     8 +-----+----+
 *            |         |          |
 *            | 2     7 |         8.2k (muß je nach LDR angepaßt werden)
 *            |         |          |
 *            | 3     6 +----------+
 *            |         |          |
 *        +---+ 4     5 +---+      |
 *        |   |         | Piezo   LDR
 *        |   +---------+ Speaker  |
 *        |                 |      |
 *       ---               ---    ---
 *
 *  AVR Memory Usage
 *  ----------------
 *  Device: attiny13
 *
 *  Program:     988 bytes (96.5% Full)
 *  (.text + .data + .bootloader)
 *
 *  Data:         12 bytes (18.8% Full)
 *  (.data + .bss + .noinit)
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>

#define MIN	10						// f_min = 1.2kHz
#define MAX	230						// f_max = 2.14kHz

int z, zalt, dz;						// Variable für Zufallszahlen
int i;								// Laufvariable

int main(void){
	// Clock Division Factor = 2 (F_CPU = 4.8 MHz)
	CLKPR = 1<<CLKPCE;
	CLKPR = 1<<CLKPS0;
	
	// PB0 = OC0A PWM Output Port (Piezo Speaker)
	DDRB = 1<<PB0;
	
	// Timer0: Phase Correct PWM Mode
	// Set OC0A on Compare Match when up-counting, Clear OC0A on Compare
	// Match when down-counting
	// TOP = 0xFF, Compare Value = OCR0A, Preset Value = z
	TCCR0A = (1<<COM0A1)|(1<<COM0A0)|(1<<WGM00);
	TIMSK0 = 1<<TOIE0;					// Timer0 Overflow Interrupt Enable
	ACSR = 1<<ACBG;					// Bandgap Reference is input to
									// the Analog Comparator
	
	sei();
     srand(10);						// seed

	for (;;) {
		if (ACSR & (1<<ACO)) {			// Ist es hell, dann
			TCCR0B = 0;				// ist die Mücke stumm (Timer0 off)
		}
		else {
			TCCR0B = 1<<CS01;			// Dunkel: Divide Timer0_Clock by 8
		}
		
		OCR0A = 249 + rand()%6;			// Zufällige Impulsbreite (Lautstärke)
		z = (rand()%((MAX+1)-MIN))+MIN;	// Zufällige Tonhöhe (Timer0 Preset)
		dz = z-zalt;
		if (dz >= 0) {					// langsame Änderung der Tonhöhe
	 		for (i = 0; i < dz; i++) {	//     von tief nach hoch
				z = zalt+i;
				_delay_ms(5);
			}
		}
		else {
			for (i = 0; i > dz; i--) {	// ... bzw. von hoch nach tief
				z = zalt + i;
				_delay_ms(2);
			}
		}
		zalt = z;
	}
	return(0);
}

ISR(TIM0_OVF_vect){						// Setzen der (neuen) Tonhöhe
	TCNT0 = z;
}